/*
** Module   : GuestAccess.java
** Abstract : provides open access to the specified P2J account
**
** Copyright (c) 2008-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ---------------------------- Description ----------------------------
** 001 NVS 20081418   @38031 Created the initial version. This class is
**                           a custom P2J authenticator. It provides two
**                           sides: client side login logic and the server
**                           side authentication hook that handles the
**                           input. Both provide unchecked access to the
**                           user account specified with the parameter.
** 002 SVL 20100923          Changed to conform the new Authenticator API.
** 003 LMR 20101105          Changed clientAuthHook() to look up "option" 
**                           entry instead of "subjectId" in the parameter 
**                           map ("subjectId" is used with a different 
**                           plug-in).
** 004 OM  20131018          Updated to conform the new Authenticator API.
*/

package com.goldencode.p2j.security;

import com.goldencode.p2j.net.SessionListener;
import java.io.*;
import java.util.*;

/**
 * The class to request authentication to a specified user account without
 * providing password.
 */
public class GuestAccess
implements Authenticator
{
   /** The value of "option" node from AuthPlugin for this Authenticator */
   private String authOption = null;

   /**
    * Default constructor. 
    */
   public GuestAccess()
   {
      // nothing to do here
   }

   /**
    * Implements client side custom authentication logic.
    *
    * @param    parameters
    *           Additional configuration parameters. This plugin uses
    *           "subjectId" parameter which corresponds the name of the user
    *           account to be used.
    * @param    code
    *           The result of the most recent attempt to authenticate or
    *           <code>AUTH_RESULT_NONE</code> if this is the first attempt.
    *
    * @return   Array of bytes to be transmitted to the server as the
    *           authorization input. 
    */
   @Override
   public byte[] clientAuthHook(Map<String, Object> parameters, int code)
   {
      String id = "";
      if (parameters != null)
         id = (String) parameters.get("option");
      
      if (id == null)
         id = "";
      
      String pw = "";

      // return a byte array
      return SecurityManager.packageIdPassword(id, pw);      
   }
   
   /**
    * Finalizes any resources allocated during authentication by the client.
    */
   @Override
   public void clientFinalize()
   {
      // nothing to do here
   }

   /**
    * Not used for the client side.
    *
    * @param    auth
    *           The authorization input from the client.
    * @param    entity
    *           Additional configuration parameters taken from the directory.
    *
    * @return   Always <code>null</code>.
    */
   @Override
   public AuthenticationResponse serverAuthHook(byte[] auth, String entity)
   {
      // parse the byte array input
      ByteArrayInputStream bis = new ByteArrayInputStream(auth);
      DataInputStream      dis = new DataInputStream(bis);
      
      String userId = null;
      
      try
      {
         userId = dis.readUTF();
      }
      catch (IOException ioe)
      {
         return new AuthenticationResponse(null, AUTH_RESULT_UNSPECIFIED_FAILURE);
      }
      
      // Safely query cached data
      SecurityCache sc =  SecurityManager.getInstance().getCache(); 

      // locate the account and its password hash
      Account acc = sc.getAccountById(userId);
      
      if (acc == null)
      {
         return new AuthenticationResponse(null, AUTH_RESULT_INVALID_USERID);
      }
      
      if (acc.getAccountType() != Account.ACC_USER)
      {
         return new AuthenticationResponse(null, AUTH_RESULT_UNSPECIFIED_FAILURE);
      }
      
      UserAccount user = (UserAccount) acc;
      
      // no password verification 
      return new AuthenticationResponse(userId, AUTH_RESULT_SUCCESS);
   }

   /**
    * Always returns <code>null</code>.
    *
    * @return  Always <code>null</code>.
    */
   @Override
   public SessionListener getSessionListener()
   {
      return null;
   }

   /**
    * Returns a set of entities that this class handles. The list of entities / parameters will
    * be configured using other ways.
    *
    * @return  A set with the value of option that was configured or an empty set if option was
    *          not configured.
    */
   @Override
   public Set<String> getAuthenticationEntities()
   {
      Set<String> ret = new HashSet<>();
      if (authOption != null)
      {
         ret.add(authOption);
      }
      return ret;
   }

   /**
    * Configures the Authenticator by setting the "option" parameter from directory.xml.
    *
    * @param   option
    *          The value of "option" entry for the auth plugin.
    */
   @Override
   public void configure(String option)
   {
      this.authOption = option;
   }
}
